
public class Bank {
	
	
	private static final int maxAccount = 10;
	private static final int initAccountCount = 3;
	
	public Bank() {
		
		account = new int[maxAccount];
		suspended = new boolean[maxAccount];
		
		/*
		 * The initialize function adds 3 accounts to bank.
		 */
		initialize();
	}
	
	/*
	 * The function accepts the ID of the account as argument and stores it in 
	 * the account array. 
	 * 
	 * Pre-Condition -
	 *	Account ID must not be zero.
	 *	Account space must not be full.
	 *	Account must not already exist.
	 * 
	 * Post-Condition -
	 *	Account data is stored in an empty cell.
	 *	Data of other accounts must not changed
	 */
	
	/*@ normal_behaviour
	@   requires (acc != 0);
	@   requires (\exists int k; 0 <= k && k < account.length; 
	@			account[k] == 0);
	@   requires (\forall int k; 0 <= k && k < account.length; 
	@			account[k] != acc);
	@   ensures (\exists int k; 0 <= k && k < account.length; 
	@			\old(account[k]) == 0 && account[k] == \old(acc) &&
	@			(\forall int m; 0 <= m && m < account.length && m != k; 
	@			\old(account[m]) == account[m]));
	@   assignable account[*];
	@*/
	public void addAccount(int acc) {
		account[find(account,0)] = acc;
	}
	
	
	/*
	 * The function accepts account ID as arguments
	 * and marks the account as suspended.
	 * 
	 * Pre-Condition -
	 *	account ID must not be zero.
	 *	account must exist.
	 *	account must not be suspended already.
	 * 
	 * 
	 * Post-Condition -
	 *	The suspended status of account is marked as true
	 *	The suspended status of other accounts must not change.
	 *  
	 */
	
	/*@ normal_behaviour
	@	requires (acc != 0);
	@	requires (\exists int k; 0 <= k && k < account.length;
	@			account[k] == acc && !suspended[k]);
	@	ensures (\exists int k; 0 <= k && k < account.length;
	@			account[k] == \old(acc) && suspended[k] &&
	@			(\forall int m; 0 <= m && m < suspended.length && m != k;
	@			\old(suspended[m]) == suspended[m])); 
	@	assignable suspended[*];
	@*/
	
	public void suspendAccount(int acc) {
		suspended[find(account,acc)] = true;
	}

	
	
	/*
	 * The function finds a number (key) in an int array. 
	 * It returns the length of the array if key is not found
	 * else returns the index of key.
	 * 
	 * Pre-Condition -
	 *	Array must not be null.
	 *	Array must not be empty.
	 * 
	 * Post-Condition -
	 *	Return value must not be greater than array size
	 *	Return array size if the key is not found
	 *	Return array index of the key if it is found.
	 */
	
	/*@ normal_behaviour
	@	requires a != null;
	@	requires a.length > 0;
	@	ensures \result <= a.length;
	@	ensures \result == a.length ==> 
	@			(\forall int k; 0 <= k && k < a.length; a[k] != \old(key));
	@	ensures \result < a.length ==> a[\result] == \old(key);
	@	assignable \nothing;
	@*/
	protected int find(int[] a, int key) {
		
		int i = 0;
		
		/*@ loop_invariant
		@	(0 <= i && i <= a.length) &&
		@	(\forall int k; 0 <= k && k < i; a[k] != key);
		@	assignable i;
		@	decreases a.length - i;
		@*/
		while(i < a.length && a[i] != key)
			i++;
		
		return i;
	}
	
	protected void initialize() {

		// Add books to Library
		
		for(int i = 0; i < initAccountCount ; i++) {
			account[i] = i+1;
		}
			
	}
	
	private /*@ spec_public @*/ int[] account;
	private /*@ spec_public @*/ boolean[] suspended;

	public static void main(String[] args) {
		// TODO Auto-generated method stub

	}

}
